#!/bin/bash

#Get the name of the VirtualBox machine the session is created for. This is needed to identify the screen session and its PID.
vboxmachine=$1

if [ `whereis Xdialog | sed 's/ /\n/g' | grep bin -c` -gt 0 ]
then
	#Create a dialog with 
	Xdialog --begin 800 550 --cancel-label "KILL" --title "VNC session termination" --infobox "The session will be ended completely and cannot
be resumed later if you press KILL. All data in
this session will be destroyed!!!" 9 50 0
	echo -n 1 > /tmp/VNCSessionTerminationCancel.code
else
	zenity --info --text "The session will be ended completely and cannot
be resumed later if you press OK. All data in
this session will be destroyed!!!" --title "VNC session termination"
	echo -n 0 > /tmp/VNCSessionTerminationCancel.code
fi

if [ $? -eq `cat /tmp/VNCSessionTerminationCancel.code` ]
then
	#Wipe out killed screen sessions (This will only clean previous killed sessions and not the current)
	screen -wipe

	#Get the number of the current X session. This is identically with the VNC screen number.
	sessionNumber=`echo $DISPLAY | cut -d':' -f2 | cut -d'.' -f1`
	#Get the PID of the XVnc4 process for this session
	vncPID=`cat /tmp/.X$sessionNumber""-lock | tr -d [:blank:]`

	#Remove the lock file to free the session number
	rm /tmp/.X$sessionNumber""-lock /tmp/.X11-unix/X$sessionNumber -f

	#Get the PID of the screen all is executed under
	screenPID=`screen -ls | grep "m23VBoxVNC-$vboxmachine" | tr -d [:blank:] | cut -d'.' -f1`	

	#Kill all tools started by VBoxVNCStarter identified by their pids
	for pid in `cat /tmp/m23VBoxVNCPIDs-$vboxmachine` $vncPID $screenPID
	do
		kill -9 $pid
	done
fi
